/****************************************************************************
 *
 * Rohde & Schwarz CMW Communication tester Base instrument driver include
 * file
 ****************************************************************************/

#ifndef __RSCMW_HEADER
#define __RSCMW_HEADER

#include "rscore.h"
#include "rscmw_attributes.h"
#include "rscmw_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSCMW_SIMULATION_ID_QUERY      "Rohde&Schwarz,CMW,123456/789,1.23"  /* Default definition of ID Query Response for simulation */
#define RSCMW_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSCMW_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSCMW_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSCMW_CLASS_DRIVER_DESCRIPTION           ""
#define RSCMW_CLASS_DRIVER_PREFIX                ""
#define RSCMW_CLASS_DRIVER_VENDOR                ""
#define RSCMW_CLASS_DRIVER_REVISION              ""
#define RSCMW_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSCMW_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSCMW_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSCMW_SPECIFIC_DRIVER_PREFIX             "RSCMW"
#define RSCMW_SPECIFIC_DRIVER_LOCATOR            ""
#define RSCMW_SPECIFIC_DRIVER_VENDOR             ""
#define RSCMW_MAJOR_VERSION                      3L    /* Instrument driver major version          */
#define RSCMW_MINOR_VERSION                      7L   /* Instrument driver minor version          */
#define RSCMW_MINOR_MINOR_VERSION                401L    /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSCMW_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSCMW_GROUP_CAPABILITIES                 ""
#define RSCMW_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSCMW_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSCMW_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSCMW_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSCMW_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSCMW_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSCMW_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSCMW_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSCMW_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSCMW_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSCMW_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSCMW_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSCMW_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSCMW_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSCMW_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSCMW_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSCMW_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSCMW_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSCMW_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSCMW_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSCMW_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSCMW_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

/* BEGIN GENERATE */
#define RSCMW_ATTR_SYSTEM_REFERENCE_FREQUENCY_SOURCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSCMW_ATTR_SYSTEM_REFERENCE_EXTERNAL_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSCMW_ATTR_REFERENCE_FREQUENCY_STATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSCMW_ATTR_INTERNAL_TEMPERATURE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 95
#define RSCMW_ATTR_SYNCHRONIZATION_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSCMW_ATTR_CONFIGURE_NETWORK_HOSTNAME                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSCMW_ATTR_CONFIGURE_NETWORK_DHCP                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSCMW_ATTR_CONFIGURE_NETWORK_ADAPTER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSCMW_ATTR_ENABLE_DNS_ADDRESS_CONFIGURATION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSCMW_ATTR_CONFIGURE_NETWORK_IP_ADDRESS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSCMW_ATTR_CONFIGURE_NETWORK_SUBNET_MASK                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSCMW_ATTR_CONFIGURE_NETWORK_GATEWAY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSCMW_ATTR_SYSTEM_GPIB_ADDRESS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSCMW_ATTR_SYSTEM_ENABLE_GPIB_INTERFACE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSCMW_ATTR_SYSTEM_ENABLE_VXI_INTERFACE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSCMW_ATTR_CONFIGURE_SUBINSTRUMENTS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSCMW_ATTR_MAX_NUMBER_OF_SUBINSTRUMENTS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define RSCMW_ATTR_ASSIGN_RESOURCES_TO_SUBINSTRUMENTS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define RSCMW_ATTR_DEVICE_ID                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSCMW_ATTR_CMWS_DEVICE_ID                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 109
#define RSCMW_ATTR_DEVICE_GROUP                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define RSCMW_ATTR_STARTUP_BEHAVIOR                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 89
#define RSCMW_ATTR_CONFIGURE_FONT_SIZE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSCMW_ATTR_CONFIGURE_UPDATE_ON_REMOTE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSCMW_ATTR_MONITOR_DISPLAY_STATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSCMW_ATTR_TREE_NAVIGATION_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSCMW_ATTR_CLEAR_ALL_EVENT_STATUS_REGISTER                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSCMW_ATTR_CLEAR_EVENT_STATUS_REGISTER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSCMW_ATTR_PRESET_STATUS_REGISTERS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSCMW_ATTR_STATUS_OQ_CONFIGURATION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSCMW_ATTR_STATUS_OQ_TASK_CONFIGURATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSCMW_ATTR_STATUS_OQ_TASK_INSTRUMENT_PART_CONFIGURE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSCMW_ATTR_STATUS_OQ_TASK_NETSTD_PART_CONFIGURE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSCMW_ATTR_STATUS_OQ_TASK_FUNCTION_GROUP_PART_CONFIGURE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSCMW_ATTR_STATUS_OQ_TASK_PART_CONFIGURE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSCMW_ATTR_STATUS_OQ_REGISTER_BIT_CONFIGURATION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSCMW_ATTR_STATUS_ERROR                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSCMW_ATTR_MACRO_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 99
#define RSCMW_ATTR_DELETE_ALL_MACROS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 100
#define RSCMW_ATTR_CALIBRATION_ALL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSCMW_ATTR_RF_CORRECTION_FILE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define RSCMW_ATTR_CLEAR_BUFFER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSCMW_ATTR_CONTINUE_BUFFER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSCMW_ATTR_DELETE_BUFFER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSCMW_ATTR_START_BUFFER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSCMW_ATTR_STOP_BUFFER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSCMW_ATTR_SET_SYSTEM_TIMESTAMP                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSCMW_ATTR_GET_SYSTEM_TIMESTAMP                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSCMW_ATTR_SET_ABSOLUTE_TIMER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSCMW_ATTR_SET_RELATIVE_TIMER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSCMW_ATTR_SUBNET_NODE_NAME                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSCMW_ATTR_SUBNET_NODE_TYPE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSCMW_ATTR_NETWORK_ADAPTER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSCMW_ATTR_SUBNET_MONITOR_NAME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSCMW_ATTR_SUBNET_MONITOR_TYPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSCMW_ATTR_SUBNET_MONITOR_ID                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define RSCMW_ATTR_SUBNET_MONITOR_DESCRIPTION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 73
#define RSCMW_ATTR_SUBNET_MONITOR_REFRESH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define RSCMW_ATTR_REMOTE_TRACE_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 101
#define RSCMW_ATTR_REMOTE_TRACE_FORMAT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 102
#define RSCMW_ATTR_REMOTE_TRACE_START_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 103
#define RSCMW_ATTR_REMOTE_TRACE_STOP_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 104
#define RSCMW_ATTR_REMOTE_TRACE_FILE_PATH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 105
#define RSCMW_ATTR_REMOTE_TRACE_FILE_SIZE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 106
#define RSCMW_ATTR_CONFIGURE_REPORT_DISPLAY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSCMW_ATTR_CLEAR_REMOTE_TRACE_DISPLAY                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 81
#define RSCMW_ATTR_GET_RESOURCE_STRING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSCMW_ATTR_CONFIGURE_LOCK_CTRLS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSCMW_ATTR_SET_BASE_TRIGGER_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSCMW_ATTR_SET_BASE_TRIGGER_SOURCE_DIRECTION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSCMW_ATTR_SET_BASE_TRIGGER_SLOPE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSCMW_ATTR_BASE_TRIGGER_SOURCE_CATALOG                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSCMW_ATTR_INITIATE_USER_TRIGGER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSCMW_ATTR_SYSTEM_OPTION_LIST                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSCMW_ATTR_COMMON_RELIABILITY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSCMW_ATTR_ALL_GENERATORS_OFF                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSCMW_ATTR_ALL_MEASUREMENTS_OFF                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSCMW_ATTR_ALL_SIGNALING_APPLICATIONS_OFF                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSCMW_ATTR_FAN_CONTROL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSCMW_ATTR_MULTIPLE_WINDOW_MODE_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 107
#define RSCMW_ATTR_FD_LIST_OF_TABLE_NAMES                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSCMW_ATTR_DELETE_ATTENUATION_CORRECTION_TABLE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSCMW_ATTR_DELETE_ALL_ATTENUATION_CORRECTION_TABLE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 82
#define RSCMW_ATTR_SAVE_ATTENUATION_CORRECTION_TABLE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 83
#define RSCMW_ATTR_LOAD_ATTENUATION_CORRECTION_TABLE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define RSCMW_ATTR_DEACTIVATE_CORRECTION_TABLE_FOR_CONNECTOR                RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSCMW_ATTR_DEACTIVATE_CORRECTION_TABLE_FOR_CONNECTOR_CMWS           RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define RSCMW_ATTR_ATTENUATION_CORRECTION_TABLE_COUNT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSCMW_ATTR_SYSTEM_VERSION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSCMW_ATTR_SYSTEM_ERROR                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSCMW_ATTR_SYSTEM_ALL_ERRORS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSCMW_ATTR_SYST_ERR_CODE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSCMW_ATTR_SYSTEM_ALL_ERRORS_CODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSCMW_ATTR_SYSTEM_PRESET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSCMW_ATTR_SYSTEM_PRESET_APPLICATION                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 87
#define RSCMW_ATTR_SYSTEM_RESET_APPLICATION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSCMW_ATTR_SYSTEM_RESET_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 110
#define RSCMW_ATTR_SYSTEM_RESET_BASE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 111
#define RSCMW_ATTR_RCL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSCMW_ATTR_SAV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSCMW_ATTR_FILE_MANAGER_EDIT_PATH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSCMW_ATTR_FILE_MANAGER_DELETE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSCMW_ATTR_FILE_MANAGER_MAKE_DIRECTORY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSCMW_ATTR_FILE_MANAGER_DELETE_DIRECTORY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSCMW_ATTR_FILE_MANAGER_GET_INSTRUMENT_DRIVES                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define RSCMW_ATTR_FILE_MANAGER_CHANGE_DRIVE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSCMW_ATTR_FILE_MANAGER_ALIASES                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSCMW_ATTR_FILE_CAPTURE_INTERIOR_WINDOW                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 93
#define RSCMW_ATTR_FILE_CAPTURE_ENTIRE_WINDOW                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 94
#define RSCMW_ATTR_HARDCOPY_FILE_FORMAT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 108
#define RSCMW_ATTR_HARDCOPY_FILE_AREA                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 112
#define RSCMW_ATTR_ID_QUERY_RESPONSE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 66

// rscmw_rngIntExt
#define RSCMW_VAL_INT                                                       0
#define RSCMW_VAL_EXT                                                       1

// rscmw_rngDirection
#define RSCMW_VAL_DIRECTION_IN                                              0
#define RSCMW_VAL_DIRECTION_OUT                                             1

// rscmw_rngFontSize
#define RSCMW_VAL_FONT_SMALL                                                0
#define RSCMW_VAL_FONT_LARGE                                                1

// rscmw_rngDeactivateConnector
#define RSCMW_VAL_CON_RF1OUT                                                0
#define RSCMW_VAL_CON_RF1COM                                                1
#define RSCMW_VAL_CON_RF2COM                                                2
#define RSCMW_VAL_CON_RF3OUT                                                3
#define RSCMW_VAL_CON_RF3COM                                                4
#define RSCMW_VAL_CON_RF4COM                                                5
#define RSCMW_VAL_CON_RFAOUT                                                6
#define RSCMW_VAL_CON_RFACOM                                                7
#define RSCMW_VAL_CON_RFBCOM                                                8

// rscmw_rngSyncMode
#define RSCMW_VAL_MODE_LIST                                                 0
#define RSCMW_VAL_MODE_GEN                                                  1
#define RSCMW_VAL_MODE_STAN                                                 2

// rscmw_rngReportDisplay
#define RSCMW_VAL_REPORT_DISPLAY_ANALYSIS                                   0
#define RSCMW_VAL_REPORT_DISPLAY_LIVE                                       1
#define RSCMW_VAL_REPORT_DISPLAY_OFF                                        2

// rscmw_rngFanControl
#define RSCMW_VAL_FAN_LOW                                                   0
#define RSCMW_VAL_FAN_NORMAL                                                1
#define RSCMW_VAL_FAN_HIGH                                                  2

// rscmw_rngSetBaseTriggerSlope
#define RSCMW_VAL_SLOPE_RISING                                              0
#define RSCMW_VAL_SLOPE_FALLING                                             1

// rscmw_rngDeactivateCorrectionTableForConnectorCMWS
#define RSCMW_VAL_CON_R11                                                   0
#define RSCMW_VAL_CON_R12                                                   1
#define RSCMW_VAL_CON_R13                                                   2
#define RSCMW_VAL_CON_R14                                                   3
#define RSCMW_VAL_CON_R15                                                   4
#define RSCMW_VAL_CON_R16                                                   5
#define RSCMW_VAL_CON_R17                                                   6
#define RSCMW_VAL_CON_R18                                                   7
#define RSCMW_VAL_CON_R21                                                   8
#define RSCMW_VAL_CON_R22                                                   9
#define RSCMW_VAL_CON_R23                                                   10
#define RSCMW_VAL_CON_R24                                                   11
#define RSCMW_VAL_CON_R25                                                   12
#define RSCMW_VAL_CON_R26                                                   13
#define RSCMW_VAL_CON_R27                                                   14
#define RSCMW_VAL_CON_R28                                                   15
#define RSCMW_VAL_CON_R31                                                   16
#define RSCMW_VAL_CON_R32                                                   17
#define RSCMW_VAL_CON_R33                                                   18
#define RSCMW_VAL_CON_R34                                                   19
#define RSCMW_VAL_CON_R35                                                   20
#define RSCMW_VAL_CON_R36                                                   21
#define RSCMW_VAL_CON_R37                                                   22
#define RSCMW_VAL_CON_R38                                                   23
#define RSCMW_VAL_CON_RA1                                                   24
#define RSCMW_VAL_CON_RA2                                                   25
#define RSCMW_VAL_CON_RA3                                                   26
#define RSCMW_VAL_CON_RA4                                                   27
#define RSCMW_VAL_CON_RA5                                                   28
#define RSCMW_VAL_CON_RA6                                                   29
#define RSCMW_VAL_CON_RA7                                                   30
#define RSCMW_VAL_CON_RA8                                                   31
#define RSCMW_VAL_CON_RB1                                                   32
#define RSCMW_VAL_CON_RB2                                                   33
#define RSCMW_VAL_CON_RB3                                                   34
#define RSCMW_VAL_CON_RB4                                                   35
#define RSCMW_VAL_CON_R118                                                  36
#define RSCMW_VAL_CON_R214                                                  37
#define RSCMW_VAL_CON_R318                                                  38
#define RSCMW_VAL_CON_R258                                                  39
#define RSCMW_VAL_CON_RA18                                                  40
#define RSCMW_VAL_CON_RB14                                                  41

// rscmw_rngTreeNavigationMode
#define RSCMW_VAL_TRAVERSE_TREES_ZIGZAG                                     0
#define RSCMW_VAL_TRAVERSE_TREES_CURSORS                                    1
#define RSCMW_VAL_TRAVERSE_TREES_VERTICAL                                   2

// rscmw_rngHcopyFormat
#define RSCMW_VAL_HCOPY_BMP                                                 0
#define RSCMW_VAL_HCOPY_JPG                                                 1
#define RSCMW_VAL_HCOPY_PNG                                                 2

// rscmw_rngTraceFormat
#define RSCMW_VAL_TRACE_FORMAT_ASCII                                        0
#define RSCMW_VAL_TRACE_FORMAT_XML                                          1

// rscmw_rngRemoteTraceStopMode
#define RSCMW_VAL_REMOTE_TRACE_AUTO                                         0
#define RSCMW_VAL_REMOTE_TRACE_EXPLICIT                                     1
#define RSCMW_VAL_REMOTE_TRACE_ERROR                                        2
#define RSCMW_VAL_REMOTE_TRACE_BUFFERFULL                                   3

// rscmw_rngHardcopyArea
#define RSCMW_VAL_HCOPY_ACTIVE                                              0
#define RSCMW_VAL_HCOPY_MAIN                                                1
#define RSCMW_VAL_HCOPY_FULL_SCREEN                                         2
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSCMW_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSCMW_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSCMW_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSCMW_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/
#define RSCMW_VAL_GENMODE_LIST 3

#define RSCMW_VAL_IP_INSTR                      0
#define RSCMW_VAL_IP_MASK                       1
#define RSCMW_VAL_IP_GATEWAY                    2

#define RSCMW_VAL_PRESET_STAT_REG               0
#define RSCMW_VAL_QUEST_STAT_REG                1
#define RSCMW_VAL_OPER_STAT_REG                 2

#define RSCMW_VAL_REG_CONDITION                 0
#define RSCMW_VAL_REG_EVENT                     1
#define RSCMW_VAL_REG_ENABLE                    2
#define RSCMW_VAL_REG_PTRANSITION               3
#define RSCMW_VAL_REG_NTRANSITION               4

#define RSCMW_VAL_FILE_NEW                      0
#define RSCMW_VAL_FILE_COPY                     1
#define RSCMW_VAL_FILE_RENAME                   2
#define RSCMW_VAL_FILE_DELETE                   3
#define RSCMW_VAL_FILE_RDIR                     4
#define RSCMW_VAL_FILE_CDIR                     5
#define RSCMW_VAL_FILE_CDRIVE                   6

#define RSCMW_VAL_SAVE                          0
#define RSCMW_VAL_RECALL                        1
#define RSCMW_VAL_MMEM_CATALOG_FILES            0
#define RSCMW_VAL_MMEM_CATALOG_SUBDIR           1


#define RSCMW_VAL_REG_OPER                      0
#define RSCMW_VAL_REG_TASK                      1
#define RSCMW_VAL_REG_TASK_A                    2
#define RSCMW_VAL_REG_TASK_A_NET                3
#define RSCMW_VAL_REG_TASK_A_FUN                4
#define RSCMW_VAL_REG_TASK_A_APP                5

#define RSCMW_VAL_TASK_APP_UNI                  0
#define RSCMW_VAL_TASK_APP_MEV                  1
#define RSCMW_VAL_TASK_APP_EPS                  2
#define RSCMW_VAL_TASK_APP_POW                  3
#define RSCMW_VAL_TASK_APP_IQV                  4
#define RSCMW_VAL_TASK_APP_IQR                  5


#define RSCMW_VAL_TASK_NET_GPRF                 1
#define RSCMW_VAL_TASK_NET_GSM                  2
#define RSCMW_VAL_TASK_NET_WCDM                 3
#define RSCMW_VAL_TASK_NET_EVDO                 4
#define RSCMW_VAL_TASK_NET_CDMA                 5
#define RSCMW_VAL_TASK_NET_TDSC                 6

#define RSCMW_VAL_REG_PART_S_ENAB               0
#define RSCMW_VAL_REG_PART_S_NTR                1
#define RSCMW_VAL_REG_PART_S_PTR                2

#define RSCMW_VAL_REG_OV_OFF                    0
#define RSCMW_VAL_REG_OV_ON                     1
#define RSCMW_VAL_REG_OV_PEN                    2
#define RSCMW_VAL_REG_OV_QUE                    3
#define RSCMW_VAL_REG_OV_RDY                    4
#define RSCMW_VAL_REG_OV_RUN                    5
#define RSCMW_VAL_REG_OV_SDR                    6

#define RSCMW_VAL_ATT_TAB_CRE                   0
#define RSCMW_VAL_ATT_TAB_ADD                   1
#define RSCMW_VAL_ATT_TAB_DEL                   2
#define RSCMW_VAL_ATT_TAB_ERASE                 3
#define RSCMW_VAL_ATT_TAB_DEL_ALL               4
#define RSCMW_VAL_ATT_TAB_SAVE                  5
#define RSCMW_VAL_ATT_TAB_LOAD                  6

#define RSCMW_VAL_ATT_TAB_ACT                   0
#define RSCMW_VAL_ATT_TAB_DEACT                 1
#define RSCMW_VAL_ATT_TAB_DEACT_ALL             2

#define RSCMW_VAL_DIRECTION_RX                  0
#define RSCMW_VAL_DIRECTION_TX                  1
#define RSCMW_VAL_DIRECTION_RT                  2

#define RSCMW_VAL_INSTR_PART_A                  1
#define RSCMW_VAL_INSTR_PART_B                  2

#define RSCMW_VAL_INSTR_CON_A                   0
#define RSCMW_VAL_INSTR_CON_B                   1

#define RSCMW_VAL_INSTR_GPIB                    0
#define RSCMW_VAL_INSTR_VXI                     1
#define RSCMW_VAL_INSTR_USB                     2

#define RSCMW_VAL_RST                           0
#define RSCMW_VAL_RST_BASE                      1
#define RSCMW_VAL_RST_ALL                       2

#define RSCMW_VAL_TIME                          0
#define RSCMW_VAL_DATE                          1

#define RSCMW_VAL_TASK_GRP_MEAS                 0
#define RSCMW_VAL_TASK_GRP_GEN                  1

#define RSCMW_VAL_REG_PART_COND                 0
#define RSCMW_VAL_REG_PART_EVENT                1

#define RSCMW_VAL_REG_OV_ALL                    0
#define RSCMW_VAL_REG_OV_CNT                    1
#define RSCMW_VAL_REG_OV_NXT                    2
#define RSCMW_VAL_REG_OV_CAT                    3

#define RSCMW_VAL_OPTION_TYPE_ALL               0
#define RSCMW_VAL_OPTION_TYPE_SWOPTION          1
#define RSCMW_VAL_OPTION_TYPE_HWOPTION          2
#define RSCMW_VAL_OPTION_TYPE_SWPACKAGES        3
#define RSCMW_VAL_OPTION_TYPE_FWA               4

#define RSCMW_VAL_VALIDITY_ALL                  0
#define RSCMW_VAL_VALIDITY_VALID                1
#define RSCMW_VAL_VALIDITY_FUNCTIONAL           2

#define RSCMW_VAL_CAL_NONE                      0
#define RSCMW_VAL_CAL_FSC                       1
#define RSCMW_VAL_CAL_UCOR                      2
#define RSCMW_VAL_CAL_CAL                       3
#define RSCMW_VAL_CAL_OGC                       4

#define RSCMW_VAL_BUFF_START                    0
#define RSCMW_VAL_BUFF_STOP                     1
#define RSCMW_VAL_BUFF_CLEAR                    2
#define RSCMW_VAL_BUFF_DELETE                   3
#define RSCMW_VAL_BUFF_CONTINUE                 4

#define RSCMW_VAL_TIM_ABS                       0
#define RSCMW_VAL_TIM_REL                       1

#define RSCMW_VAL_MEM_SAVE                      0
#define RSCMW_VAL_MEM_RECALL                    1

#define RSCMW_VAL_NETWORK_SEGMENT_A             0
#define RSCMW_VAL_NETWORK_SEGMENT_B             1
#define RSCMW_VAL_NETWORK_SEGMENT_C             2

#define RSCMW_VAL_CONVERTER_RF1                 1
#define RSCMW_VAL_CONVERTER_RF2                 2
#define RSCMW_VAL_CONVERTER_RF3                 3
#define RSCMW_VAL_CONVERTER_RF4                 4

#define RSCMW_VAL_INPUT_RX                      0
#define RSCMW_VAL_OUTPUT_TX                     1
#define RSCMW_VAL_IO_ALL                        2

#define RSCMW_VAL_CAPTURE_WINDOW_ENTIRE         0
#define RSCMW_VAL_CAPTURE_WINDOW_INTERIOR       1
/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rscmw_init (ViRsrc resourceName, ViBoolean IDQuery,
                              ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rscmw_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                         ViBoolean resetDevice,
                                         ViString optionString,
                                         ViSession* instrumentHandle);
ViStatus _VI_FUNC rscmw_AllGeneratorsOff (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_AllMeasurementsOff (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_AllSignalingApplicationsOff (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_ConfigureReferenceOscillator (ViSession instrumentHandle,
                                                      ViInt32 reference,
                                                      ViReal64 externalFrequency);
ViStatus _VI_FUNC rscmw_GetReferenceFrequencyState (ViSession instrumentHandle,
                                                    ViBoolean* locked);
ViStatus _VI_FUNC rscmw_ConfigureSynchronizationMode (ViSession instrumentHandle,
                                                      ViInt32 mode);
ViStatus _VI_FUNC rscmw_LockControls (ViSession instrumentHandle, ViBoolean locked);
ViStatus _VI_FUNC rscmw_QueryOptionList (ViSession instrumentHandle,
                                         ViInt32 bufferSize, ViInt32 optionType,
                                         ViInt32 validity,
                                         ViChar _VI_FAR optionList[]);
ViStatus _VI_FUNC rscmw_QueryOptionVersionInfo (ViSession instrumentHandle,
                                                ViInt32 bufferSize,
                                                ViString application,
                                                ViChar _VI_FAR optionList[]);
ViStatus _VI_FUNC rscmw_QueryCommonReliability (ViSession instrumentHandle,
                                                ViInt32* reliability);
ViStatus _VI_FUNC rscmw_QueryConnectorRelation (ViSession instrumentHandle,
                                                ViInt32 connector,
                                                ViInt32* virtualConnector,
                                                ViInt32* absoluteConnector);
ViStatus _VI_FUNC rscmw_QueryInternalTemperature (ViSession instrumentHandle,
                                                  ViReal64* internalTemperature);
ViStatus _VI_FUNC rscmw_ConfigureSignalingMessageMonitoring
             (ViSession instrumentHandle, ViInt32 poolEntry, ViInt32 firstSegment,
              ViInt32 secondSegment, ViInt32 nodeID, ViInt32 localID);
ViStatus _VI_FUNC rscmw_ConfigureFanControl (ViSession instrumentHandle,
                                             ViInt32 fanControl);
ViStatus _VI_FUNC rscmw_ConfigureMultipleWindowMode (ViSession instrumentHandle,
                                                     ViBoolean enabled);
ViStatus _VI_FUNC rscmw_ConfigureAdapter (ViSession instrumentHandle,
                                          ViString networkAdapter);
ViStatus _VI_FUNC rscmw_ConfigureHostname (ViSession instrumentHandle,
                                           ViString hostname);
ViStatus _VI_FUNC rscmw_ConfigureDHCP (ViSession instrumentHandle,
                                       ViBoolean DHCPState);
ViStatus _VI_FUNC rscmw_EnableDNSAddressConfiguration (ViSession instrumentHandle,
                                                       ViBoolean DNSState);
ViStatus _VI_FUNC rscmw_ConfigureDNSAddress (ViSession instrumentHandle,
                                             ViString preferredAddress,
                                             ViString alternateAddress);
ViStatus _VI_FUNC rscmw_ConfigureIPAddress (ViSession instrumentHandle,
                                            ViInt32 IPAddressSetting,
                                            ViString IPAddress);
ViStatus _VI_FUNC rscmw_SetGPIBAddress (ViSession instrumentHandle,
                                        ViInt32 address);
ViStatus _VI_FUNC rscmw_EnableInterface (ViSession instrumentHandle,
                                         ViInt32 interface,
                                         ViBoolean interfaceState);
ViStatus _VI_FUNC rscmw_ConfigureFont (ViSession instrumentHandle,
                                       ViInt32 fontSize);
ViStatus _VI_FUNC rscmw_ConfigureUpdateOnRemote (ViSession instrumentHandle,
                                                 ViBoolean update);
ViStatus _VI_FUNC rscmw_ConfigureMonitorDisplayState (ViSession instrumentHandle,
                                                      ViBoolean turnOffMonitor);
ViStatus _VI_FUNC rscmw_ConfigureTreeNavigationMode (ViSession instrumentHandle,
                                                     ViInt32 treeNavigationMode);
ViStatus _VI_FUNC rscmw_ConfigureSubinstruments (ViSession instrumentHandle,
                                                 ViInt32 subinstruments);
ViStatus _VI_FUNC rscmw_AssignResourceToSubinstruments (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_GetSubinstrumentsInfo (ViSession instrumentHandle,
                                               ViInt32* currentSubinstrument,
                                               ViInt32* subinstrumentsCount);
ViStatus _VI_FUNC rscmw_GetMaxNumberOfSubinstruments (ViSession instrumentHandle,
                                                      ViInt32* subinstrumentsCount);
ViStatus _VI_FUNC rscmw_QueryDeviceID (ViSession instrumentHandle,
                                       ViInt32 bufferSize,
                                       ViChar _VI_FAR deviceID[]);
ViStatus _VI_FUNC rscmw_QueryCMWSDeviceID (ViSession instrumentHandle,
                                           ViInt32 bufferSize,
                                           ViChar _VI_FAR deviceID[]);
ViStatus _VI_FUNC rscmw_ConfigureDeviceGroup (ViSession instrumentHandle,
                                              ViString subinstruments);
ViStatus _VI_FUNC rscmw_ConfigureStartupBehavior (ViSession instrumentHandle,
                                                  ViBoolean bootFactoryState);
ViStatus _VI_FUNC rscmw_ClearEventStatusRegister (ViSession instrumentHandle,
                                                  ViString regularExpression);
ViStatus _VI_FUNC rscmw_SetStatusRegister (ViSession instrumentHandle,
                                           ViInt32 registerOperation,
                                           ViInt32 enable, ViInt32 PTransition,
                                           ViInt32 NTransition);
ViStatus _VI_FUNC rscmw_GetStatusRegister (ViSession instrumentHandle,
                                           ViInt32 registerOperation,
                                           ViInt32 statusRegistersQuery,
                                           ViInt32* registerValue);
ViStatus _VI_FUNC rscmw_SetStatusRegisterTask (ViSession instrumentHandle,
                                               ViInt32 registerPart,
                                               ViInt32 instrumentPart,
                                               ViInt32 networkStandard,
                                               ViInt32 subsystem,
                                               ViInt32 functionGroup,
                                               ViInt32 functionGroupInstance,
                                               ViInt32 application,
                                               ViInt32 registerValue);
ViStatus _VI_FUNC rscmw_GetStatusRegisterTask (ViSession instrumentHandle,
                                               ViInt32 registerQuestionPart,
                                               ViInt32 instrumentPart,
                                               ViInt32 networkStandard,
                                               ViInt32 subsystem,
                                               ViInt32 functionGroup,
                                               ViInt32 functionGroupInstance,
                                               ViInt32 application,
                                               ViInt32* registerValue);
ViStatus _VI_FUNC rscmw_GetStatusRegisterAllBitsTasks (ViSession instrumentHandle,
                                                       ViInt32 registerPart,
                                                       ViInt32 typeOfQuery,
                                                       ViString regularExpression,
                                                       ViInt32 bufferSize,
                                                       ViChar _VI_FAR state[]);
ViStatus _VI_FUNC rscmw_GetStatusRegisterLowestTasks (ViSession instrumentHandle,
                                                      ViInt32 functionGroup,
                                                      ViInt32 typeOfQuery,
                                                      ViString regularExpression,
                                                      ViInt32 bufferSize,
                                                      ViChar _VI_FAR state[]);
ViStatus _VI_FUNC rscmw_SetStatusRegisterBit (ViSession instrumentHandle,
                                              ViInt32 registerOperation,
                                              ViInt32 registerPart,
                                              ViInt32 bitNumber,
                                              ViBoolean bitValue);
ViStatus _VI_FUNC rscmw_GetStatusRegisterBit (ViSession instrumentHandle,
                                              ViInt32 registerOperation,
                                              ViInt32 registerPart,
                                              ViInt32 bitNumber,
                                              ViBoolean* bitValue);
ViStatus _VI_FUNC rscmw_GetStatusErrorQueueEntry (ViSession instrumentHandle,
                                                  ViInt32* errorCode,
                                                  ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rscmw_CreateMacro (ViSession instrumentHandle,
                                     ViString macroLabel, ViString macroSequence);
ViStatus _VI_FUNC rscmw_StartMacroRecording (ViSession instrumentHandle,
                                             ViString macroFile);
ViStatus _VI_FUNC rscmw_StopMacroRecording (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_DeleteMacro (ViSession instrumentHandle,
                                     ViString macroLabel);
ViStatus _VI_FUNC rscmw_DeleteAllMacros (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_SaveMacro (ViSession instrumentHandle, ViString macroLabel,
                                   ViString macroFile);
ViStatus _VI_FUNC rscmw_LoadMacro (ViSession instrumentHandle, ViString macroLabel,
                                   ViString macroFile);
ViStatus _VI_FUNC rscmw_QueryMacro (ViSession instrumentHandle, ViString macroLabel,
                                    ViUInt32 arraySize,
                                    ViChar _VI_FAR macroSequence[]);
ViStatus _VI_FUNC rscmw_QueryMacroLabels (ViSession instrumentHandle,
                                          ViUInt32 arraySize,
                                          ViChar _VI_FAR macroLabels[]);
ViStatus _VI_FUNC rscmw_ConfigureBaseTrigger (ViSession instrumentHandle,
                                              ViInt32 connector,
                                              ViString triggerSource,
                                              ViInt32 direction);
ViStatus _VI_FUNC rscmw_ConfigureBaseTriggerSlope (ViSession instrumentHandle,
                                                   ViInt32 connector,
                                                   ViInt32 slope);
ViStatus _VI_FUNC rscmw_QueryBaseTriggerSourceCatalog (ViSession instrumentHandle,
                                                       ViInt32 bufferSize,
                                                       ViInt32 connector,
                                                       ViChar _VI_FAR optionList[]);
ViStatus _VI_FUNC rscmw_InitiateUserTrigger (ViSession instrumentHandle,
                                             ViInt32 userTrigger);
ViStatus _VI_FUNC rscmw_QueryAllCalibrations (ViSession instrumentHandle,
                                              ViInt32 bufferSize,
                                              ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rscmw_QueryLatestCalibration (ViSession instrumentHandle,
                                                ViInt32 bufferSize, ViInt32 type,
                                                ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rscmw_QueryCalibrationDateTime (ViSession instrumentHandle,
                                                  ViInt32 bufferSize, ViInt32 type,
                                                  ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rscmw_QueryRFCorrectionFile (ViSession instrumentHandle,
                                               ViInt32 bufferSize,
                                               ViChar _VI_FAR result[]);
ViStatus _VI_FUNC rscmw_BufferOperations (ViSession instrumentHandle,
                                          ViInt32 operation, ViString bufferLabel);
ViStatus _VI_FUNC rscmw_FetchBuffer (ViSession instrumentHandle, ViInt32 bufferSize,
                                     ViString bufferLabel, ViInt32 lineNumber,
                                     ViChar _VI_FAR bufferContent[]);
ViStatus _VI_FUNC rscmw_FetchBufferLines (ViSession instrumentHandle,
                                          ViString bufferLabel,
                                          ViInt32* numberOfLines);
ViStatus _VI_FUNC rscmw_SetTimestamp (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_QueryTimestamp (ViSession instrumentHandle,
                                        ViInt32 bufferSize,
                                        ViChar _VI_FAR timestamp[]);
ViStatus _VI_FUNC rscmw_StartTimer (ViSession instrumentHandle, ViInt32 timer,
                                    ViInt32 timeout);
ViStatus _VI_FUNC rscmw_ConfigureSubnetNode (ViSession instrumentHandle,
                                             ViInt32 nodeID, ViString description,
                                             ViInt32 networkSegment);
ViStatus _VI_FUNC rscmw_QuerySubnetNodeInfo (ViSession instrumentHandle,
                                             ViChar _VI_FAR nodeName[],
                                             ViChar _VI_FAR nodeType[],
                                             ViInt32* selectedNetworkSegment,
                                             ViChar _VI_FAR IPAddress[],
                                             ViChar _VI_FAR subnetMask[]);
ViStatus _VI_FUNC rscmw_ConfigureNetworkAdapter (ViSession instrumentHandle,
                                                 ViInt32 index,
                                                 ViBoolean setSubnetConform);
ViStatus _VI_FUNC rscmw_QueryNetworkAdapter (ViSession instrumentHandle,
                                             ViInt32 index,
                                             ViChar _VI_FAR networkAdapterName[],
                                             ViChar _VI_FAR IPAddress[],
                                             ViInt32* status);
ViStatus _VI_FUNC rscmw_QuerySubnetMonitor (ViSession instrumentHandle,
                                            ViChar _VI_FAR nodeNames[],
                                            ViChar _VI_FAR nodeTypes[],
                                            ViChar _VI_FAR ID[],
                                            ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rscmw_RefreshSubnetMonitor (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_ConfigureRemoteTrace (ViSession instrumentHandle,
                                              ViInt32 subinstrument,
                                              ViBoolean enabled);
ViStatus _VI_FUNC rscmw_ConfigureReportDisplay (ViSession instrumentHandle,
                                                ViInt32 mode);
ViStatus _VI_FUNC rscmw_ConfigureRemoteTraceFile (ViSession instrumentHandle,
                                                  ViInt32 subinstrument,
                                                  ViString fileName,
                                                  ViInt32 fileSize, ViInt32 format);
ViStatus _VI_FUNC rscmw_ConfigureRemoteTraceFilter (ViSession instrumentHandle,
                                                    ViInt32 subinstrument,
                                                    ViInt32 arraySize,
                                                    ViBoolean _VI_FAR states[]);
ViStatus _VI_FUNC rscmw_ConfigureRemoteTraceStartStopMode
             (ViSession instrumentHandle, ViInt32 subinstrument, ViInt32 startMode,
              ViInt32 stopMode);
ViStatus _VI_FUNC rscmw_ClearSCPIRemoteTraceDisplay (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_ConfigureAttenuationCorrectionTable
             (ViSession instrumentHandle, ViInt32 tableOperation,
              ViString tableName, ViString tableData);
ViStatus _VI_FUNC rscmw_ConfigureAttenuationCorrectionTableDirection
             (ViSession instrumentHandle, ViInt32 tableOperation, ViInt32 connector,
              ViString tableName, ViInt32 direction, ViInt32 RFConverter);
ViStatus _VI_FUNC rscmw_DeactivateAllAttenuationCorrectionTables
             (ViSession instrumentHandle, ViInt32 direction, ViString tablePath);
ViStatus _VI_FUNC rscmw_QueryAttenuationCorrectionTableExistance
             (ViSession instrumentHandle, ViString tableName, ViInt32* result);
ViStatus _VI_FUNC rscmw_ConfigureCMWSAttenuationCorrectionTableDirection
             (ViSession instrumentHandle, ViInt32 tableOperation, ViInt32 direction,
              ViInt32 connector, ViString tables);
ViStatus _VI_FUNC rscmw_ConfigureCMWSGPRFGeneratorSingleRFConnector
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 connector,
              ViBoolean connectorState);
ViStatus _VI_FUNC rscmw_ConfigureCMWSGPRFGeneratorIndividualRFConnector
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 connector,
              ViInt32 bufferSize, ViBoolean _VI_FAR connectorState[]);
ViStatus _VI_FUNC rscmw_DeactivateCMWSAllAttenuationCorrectionTables
             (ViSession instrumentHandle, ViInt32 direction, ViString tablePath);
ViStatus _VI_FUNC rscmw_GetListOfAttenuationTables (ViSession instrumentHandle,
                                                    ViInt32 bufferSize,
                                                    ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTablesCount
             (ViSession instrumentHandle, ViInt32* numberOfTables);
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTable (ViSession instrumentHandle,
                                                       ViInt32 connector,
                                                       ViInt32 RFConverter,
                                                       ViInt32 bufferSize,
                                                       ViChar _VI_FAR assignedTables[]);
ViStatus _VI_FUNC rscmw_GetCMWSAttenuationCorrectionTable
             (ViSession instrumentHandle, ViInt32 connector, ViInt32 bufferSize,
              ViChar _VI_FAR assignedTables[]);
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTableDetails
             (ViSession instrumentHandle, ViString tableName, ViInt32 startIndex,
              ViInt32 count, ViInt32 bufferSize, ViChar _VI_FAR details[]);
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTableEntries
             (ViSession instrumentHandle, ViString tableName, ViInt32* entries);
ViStatus _VI_FUNC rscmw_FileManagerOperations (ViSession instrumentHandle,
                                               ViInt32 operation, ViString source,
                                               ViString destination);
ViStatus _VI_FUNC rscmw_GetCurrentDirectory (ViSession instrumentHandle,
                                             ViInt32 bufferSize,
                                             ViChar _VI_FAR currentDirectory[]);
ViStatus _VI_FUNC rscmw_GetCurrentDrive (ViSession instrumentHandle,
                                         ViInt32 bufferSize,
                                         ViChar _VI_FAR currentDrive[]);
ViStatus _VI_FUNC rscmw_GetInstrumentDrives (ViSession instrumentHandle,
                                             ViInt32 bufferSize,
                                             ViChar _VI_FAR instrumentDrives[]);
ViStatus _VI_FUNC rscmw_FileDirectoryPath (ViSession instrumentHandle, ViInt32 read,
                                           ViString directory, ViInt32 bufferSize,
                                           ViChar _VI_FAR output[],
                                           ViInt32* returnedValues);
ViStatus _VI_FUNC rscmw_FileDirectoryPathCount (ViSession instrumentHandle,
                                                ViInt32 read, ViString directory,
                                                ViInt32* numbersOfDir);
ViStatus _VI_FUNC rscmw_DataSetFileOperations (ViSession instrumentHandle,
                                               ViInt32 operation,
                                               ViInt32 stateNumber, ViString path);
ViStatus _VI_FUNC rscmw_DataSetApplicationOperations (ViSession instrumentHandle,
                                                      ViInt32 operation,
                                                      ViString itemName,
                                                      ViString path);
ViStatus _VI_FUNC rscmw_SetFileAttributes (ViSession instrumentHandle,
                                           ViString fileName, ViString action);
ViStatus _VI_FUNC rscmw_ReadFileAttributes (ViSession instrumentHandle,
                                            ViString file, ViInt32 bufferSize,
                                            ViChar _VI_FAR output[],
                                            ViInt32* returnedValues);
ViStatus _VI_FUNC rscmw_GetAliases (ViSession instrumentHandle, ViInt32 bufferSize,
                                    ViChar _VI_FAR aliases[]);
ViStatus _VI_FUNC rscmw_ConfigureScreenshotFormat (ViSession instrumentHandle,
                                                   ViInt32 format);
ViStatus _VI_FUNC rscmw_ConfigureScreenshotArea (ViSession instrumentHandle,
                                                 ViInt32 area);
ViStatus _VI_FUNC rscmw_SaveScreenshotToFile (ViSession instrumentHandle,
                                              ViInt32 location, ViString fileName);
ViStatus _VI_FUNC rscmw_SetAttributeViInt32 (ViSession instrumentHandle,
                                             ViString repeatedCapabilityName,
                                             ViUInt32 attributeID,
                                             ViInt32 attributeValue);
ViStatus _VI_FUNC rscmw_SetAttributeViReal64 (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViReal64 attributeValue);
ViStatus _VI_FUNC rscmw_SetAttributeViString (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViString attributeValue);
ViStatus _VI_FUNC rscmw_SetAttributeViBoolean (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViBoolean attributeValue);
ViStatus _VI_FUNC rscmw_SetAttributeViSession (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViSession attributeValue);
ViStatus _VI_FUNC rscmw_GetAttributeRepCapName (ViSession instrumentHandle,
                                                ViUInt32 attributeID,
                                                ViString repCapNameID,
                                                ViInt32 index, ViInt32 bufferSize,
                                                ViChar _VI_FAR repCapName[]);
ViStatus _VI_FUNC rscmw_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rscmw_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rscmw_GetAttributeViInt32 (ViSession instrumentHandle,
                                             ViString repeatedCapabilityName,
                                             ViUInt32 attributeID,
                                             ViInt32* attributeValue);
ViStatus _VI_FUNC rscmw_GetAttributeViReal64 (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViReal64* attributeValue);
ViStatus _VI_FUNC rscmw_GetAttributeViString (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViInt32 bufferSize,
                                              ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rscmw_GetAttributeViBoolean (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViBoolean* attributeValue);
ViStatus _VI_FUNC rscmw_GetAttributeViSession (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViSession* attributeValue);
ViStatus _VI_FUNC rscmw_CheckAttributeViInt32 (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rscmw_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rscmw_CheckAttributeViString (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViString attributeValue);
ViStatus _VI_FUNC rscmw_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rscmw_CheckAttributeViSession (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rscmw_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_resetApplication (ViSession instrumentHandle,
                                          ViString application);
ViStatus _VI_FUNC rscmw_self_test (ViSession instrumentHandle,
                                   ViInt16* selfTestResult,
                                   ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rscmw_revision_query (ViSession instrumentHandle,
                                        ViChar _VI_FAR instrumentDriverRevision[],
                                        ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rscmw_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                       ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rscmw_error_query (ViSession instrumentHandle, ViInt32* errorCode,
                                     ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rscmw_error_code_query (ViSession instrumentHandle,
                                          ViInt32* errorCode);
ViStatus _VI_FUNC rscmw_ErrorList (ViSession instrumentHandle, ViInt32* code,
                                   ViInt32 bufferSize, ViChar _VI_FAR errors[]);
ViStatus _VI_FUNC rscmw_error_message (ViSession instrumentHandle,
                                       ViStatus errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rscmw_storeLoadInstrumentData (ViSession instrumentHandle,
                                                 ViInt32 networkMode,
                                                 ViInt32 stateNumber);
ViStatus _VI_FUNC rscmw_Preset (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rscmw_PresetApplication (ViSession instrumentHandle,
                                           ViString application);
ViStatus _VI_FUNC rscmw_ResetDevice (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rscmw_ClearStatus (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_IDQueryResponse (ViSession instrumentHandle,
                                         ViChar _VI_FAR IDQueryResponse[]);
ViStatus _VI_FUNC rscmw_ProcessAllPreviousCommands (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_QueryOPC (ViSession instrumentHandle, ViInt32* OPC);
ViStatus _VI_FUNC rscmw_setCheckOption (ViSession instrumentHandle,
                                        ViBoolean optionChecking);
ViStatus _VI_FUNC rscmw_SetVISATimeout (ViSession instrumentHandle,
                                        ViUInt32 VISATimeout);
ViStatus _VI_FUNC rscmw_GetVISATimeout (ViSession instrumentHandle,
                                        ViUInt32* VISATimeout);
ViStatus _VI_FUNC rscmw_SetOPCTimeout (ViSession instrumentHandle,
                                       ViUInt32 OPCTimeout);
ViStatus _VI_FUNC rscmw_GetOPCTimeout (ViSession instrumentHandle,
                                       ViUInt32* timeout);
ViStatus _VI_FUNC rscmw_ConfigureErrorChecking (ViSession instrumentHandle,
                                                ViBoolean optionChecking,
                                                ViBoolean rangeChecking,
                                                ViBoolean statusChecking);
ViStatus _VI_FUNC rscmw_ReliabilityIndicator (ViSession instrumentHandle,
                                              ViInt32 reliabilityCode,
                                              ViChar _VI_FAR reliabilityMessage[]);
ViStatus _VI_FUNC rscmw_ConfigureAutoSystemErrQuery (ViSession instrumentHandle,
                                                     ViBoolean autoSystErrQuery);
ViStatus _VI_FUNC rscmw_ConfigureMultiThreadLocking (ViSession instrumentHandle,
                                                     ViBoolean multiThreadLocking);
ViStatus _VI_FUNC rscmw_GetError (ViSession instrumentHandle, ViStatus* code,
                                  ViInt32 bufferSize, ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rscmw_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmw_WriteInstrData (ViSession instrumentHandle,
                                        ViString writeBuffer);
ViStatus _VI_FUNC rscmw_WriteFromFileToInstrument (ViSession instrumentHandle,
                                                   ViString source,
                                                   ViString destination);
ViStatus _VI_FUNC rscmw_ReadInstrData (ViSession instrumentHandle,
                                       ViInt32 numberOfBytesToRead,
                                       ViChar _VI_FAR readBuffer[],
                                       ViUInt32* numBytesRead);
ViStatus _VI_FUNC rscmw_ReadToFileFromInstrument (ViSession instrumentHandle,
                                                  ViString source,
                                                  ViString destination);
ViStatus _VI_FUNC rscmw_WriteCommandWithOPCSync (ViSession instrumentHandle,
                                                 ViString writeBuffer,
                                                 ViUInt32 timeout);
ViStatus _VI_FUNC rscmw_QueryWithOPCSync (ViSession instrumentHandle,
                                          ViString writeBuffer, ViUInt32 timeout,
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR readBuffer[],
                                          ViUInt32* numBytesRead);
ViStatus _VI_FUNC rscmw_QueryViBoolean (ViSession instrumentHandle,
                                        ViString command, ViBoolean* value);
ViStatus _VI_FUNC rscmw_QueryViInt32 (ViSession instrumentHandle, ViString command,
                                      ViInt32* value);
ViStatus _VI_FUNC rscmw_QueryViReal64 (ViSession instrumentHandle, ViString command,
                                       ViReal64* value);
ViStatus _VI_FUNC rscmw_QueryViString (ViSession instrumentHandle, ViString command,
                                       ViInt32 bufferSize, ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rscmw_close (ViSession instrumentHandle);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSCMW_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSCMW_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSCMW_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSCMW_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSCMW_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)
#define RSCMW_ERROR_RELIABILITY_INDICATOR  (RS_SPECIFIC_ERROR_BASE + 0x0004L)

#define RSCMW_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSCMW_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSCMW_ERRMSG_INCORRECT_STATE       "Instrument in incorrect state"
#define RSCMW_ERRMSG_INCORRECT_RESPONSE    "Response incorrect; report problem to R&S support!"
#define RSCMW_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"
#define RSCMW_ERRMSG_RELIABILITY_INDICATOR "Reliability Indicator error"


#define RSCMW_ERROR_CODES_AND_MSGS \
    {RSCMW_ERROR_INCORRECT_STATE,   RSCMW_ERRMSG_INCORRECT_STATE},\
    {RSCMW_ERROR_INCORRECT_RESPONSE,     RSCMW_ERRMSG_INCORRECT_RESPONSE},\
    {RSCMW_ERROR_MAX_TIME_EXCEEDED,    RSCMW_ERRMSG_MAX_TIME_EXCEEDED},\
    {RSCMW_ERROR_RELIABILITY_INDICATOR, RSCMW_ERRMSG_RELIABILITY_INDICATOR}


/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/


#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSCMW_HEADER */
